/*
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/***** Includes *****/
#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>

/* Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/PIN.h>

/* Board Header files */
#include "Board.h"

#include "smartrf_settings/smartrf_settings.h"

//MS: 7.3.16
#include "sce/scif_framework.h"
#include "sce/scif_osal_tirtos.h"
#include "sce/scif.h"

#define BV(n)               (1 << (n))

#define DISABLE_LOW_POWER   0

#include <ti/drivers/UART.h>
#include <ti/drivers/uart/UARTCC26XX.h>
#include "string.h"
#include "stdio.h"
#include <driverlib/aux_adc.h> //for ADC calibration operations

#include "wmbus/mbus_packet.h"

Clock_Struct clkStruct; //FG 25.10.16 PWR
Void clkFxn(UArg arg0); //FG 25.10.16 PWR
Clock_Handle clkHandle; //FG 25.10.16 PWR


// UART driver objects
UART_Handle uHandle;
UART_Params uParams;
char pLine[21], command = '0';

// Semaphore used to wait for Sensor Controller task ALERT event
static Semaphore_Struct semScTaskAlert, semStruct, semLC;             //Semaphores for SCE and LCD
Semaphore_Handle semHandle, semLCDHandle, semLCHandle;

// Task data
#define TASKSTACKSIZE   512
Task_Struct myTask, task0Struct;
Char myTaskStack[TASKSTACKSIZE * 2], task0Stack[TASKSTACKSIZE];


// Semaphore used to wait for PCB Cap Touch task event

// Semaphore used to wait for LCD
static Semaphore_Struct semLCDOn;

// Semaphore used for LC sensing task
static Semaphore_Struct semLC;

void scCtrlReadyCallback(void);
void scTaskAlertCallback(void);
void scCtrlReadyCallback(void) {
} // scCtrlReadyCallback


void scTaskAlertCallback(void) {
    Semaphore_post(Semaphore_handle(&semScTaskAlert));     // Wake up the OS task
} // scTaskAlertCallback


extern uint8_t TXpacket[];
extern uint8_t TXbytes[];
extern uint8_t TXdataLength;
extern uint16_t  packetLength, TmodeByteLength;

//ADC calculation objects
uint32_t calADC12_gain=0, lmt70_HeatSensor=0, lmt70_RoomSensor=0, lmt70_CalInput=0;
int8_t calADC12_offset;
float temp1_LMT70lin,temp2_LMT70lin, temp1_LMT70dual,temp2_LMT70dual;
float a = -7.857923E-06, b = -1.777501E-01, c = 2.046398E+02;
uint32_t packetCounter=0, pcbTouchCounter=0, finishCount = 0;
int16_t discturns = 0;

/*******************************************************************************
*               Packet Examples from EN13757-4:2014
*   Packet S2:  0F 44 AE 0C 78 56 34 12 01 07 44 47 78 0B 13 43 65 87 1E 6D     (20 Bytes) S- and T-Modes with Format A
*   Packet C2:  14 44 AE 0C 78 56 34 12 01 07 8C 20 27 78 0B 13 43 65 87 7A C5  (21 Bytes) all other Modes with Format B
*   volume in liters: 876543
*/
//1. Feb'17 TIS = 0x5133 but order of transmission is swapped
uint8_t wMBusPacketC[]= {0x14,PACKET_C_FIELD, 0x33, 0x51, PACKET_CI_FIELD, 0x56, 0x34, 0x12, MAN_ID, MAN_VER, 0x8C, 0x20, 0x27, PACKET_CI_FIELD, 0x0B, 0x13, 0x43, 0x65, 0x87, 0x00, 0x00/*0x7A, 0xC5*/}; //C2-Mode
//volatile unsigned int  mode = 3; //C-mode
//volatile unsigned int  mode = 2; //T-mode
volatile unsigned int  mode = 0; //S-mode
extern uint16_t  packetLength, TmodeByteLength;
extern uint8_t TXbytes[];

/***** Defines *****/
#define TX_TASK_STACK_SIZE 1024
#define TX_TASK_PRIORITY   2

/* TX Configuration */
#define PAYLOAD_LENGTH      120	//was 30,; 80 and 120 Bytes work with SRF7 Studi0
#define PACKET_INTERVAL     (uint32_t)(4000000*20.0f) /* Set packet interval to 10 secs (*10.0f) */


/***** Prototypes *****/
static void txTaskFunction(UArg arg0, UArg arg1);

/***** Variable declarations *****/
static Task_Params txTaskParams;
Task_Struct txTask;    /* not static so you can see in ROV */
static uint8_t txTaskStack[TX_TASK_STACK_SIZE];

static RF_Object rfObject;
static RF_Handle rfHandle;

uint32_t time;
static uint8_t packet[PAYLOAD_LENGTH];
static PIN_Handle pinHandle;
static PIN_Handle ledPinHandle;


/***** Function definitions *****/

void TxTask_init(PIN_Handle inPinHandle)
{
    pinHandle = inPinHandle;

    Task_Params_init(&txTaskParams);
    txTaskParams.stackSize = TX_TASK_STACK_SIZE;
    txTaskParams.priority = TX_TASK_PRIORITY;
    txTaskParams.stack = &txTaskStack;
    txTaskParams.arg0 = (UInt)1000000;

    Task_construct(&txTask, txTaskFunction, &txTaskParams, NULL);
}


static void txTaskFunction(UArg arg0, UArg arg1)
{
    uint32_t time;
    RF_Params rfParams;
    RF_Params_init(&rfParams);

    RF_cmdPropTx.pktLen = PAYLOAD_LENGTH;
    RF_cmdPropTx.pPkt = packet;
    RF_cmdPropTx.startTrigger.triggerType = TRIG_ABSTIME;
    RF_cmdPropTx.startTrigger.pastTrig = 0; //FG: Packets are sent with an regular interval
    RF_cmdPropTx.startTime = 0;

    /* Request access to the radio */
    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);

    /* Set the frequency */
    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);

    /* Get current time */
    time = RF_getCurrentTime();

    /* defined in mbus_packet.h */
    mode = WMbus_mode;

    while(1)
    {
        /* 1. Create wM-Bus packet for each Mode: S, T or C */

    	uint8_t i;

    	if (mode == WMBUS_CMODE || mode == WMBUS_C2MODE || mode == WMBUS_NabefMODE || WMbus_mode == WMBUS_NcdMODE || WMbus_mode == WMBUS_NgMODE ||
    						    		 	 WMbus_mode == WMBUS_FMODE ) {
    	//	wMBusPacketC[16] = calADC12_offset; //(unsigned char) ((ui16wmbusACC_Counter&0x00FF0000)>>16);
    	//	wMBusPacketC[17] = calADC12_gain >> 8; //(unsigned char) ((ui16wmbusACC_Counter&0x0000FF00)>>8);
    		wMBusPacketC[18] = 0; //(unsigned char) (ui16wmbusACC_Counter&0x000000FF);

    	//    wMBusPacketC[17] = scifTaskData.lcsense.output.effCounter; //(unsigned char) (ui16wmbusACC_Counter&0x000000FF);
    	//    wMBusPacketC[16] = scifTaskData.lcsense.output.effCounter >> 8;

    	    discturns = scifTaskData.lcsense.output.effCounter;

    	           // wMBusPacketC[18] = wMBusPacketC[18] + 1; //(unsigned` char) (ui16wmbusACC_Counter&0x000000FF);

    	            wMBusPacketC[17] = discturns; //(unsigned char) (ui16wmbusACC_Counter&0x000000FF);
    	            wMBusPacketC[16] = discturns >> 8;

    	    /* MS 2. Oct'15: use LMT70 data
    	            packet[2] = calADC12_offset;
    	            packet[3] = calADC12_gain >> 24;
    	            packet[4] = calADC12_gain >> 16;
    	            packet[5] = calADC12_gain >> 8;
    	            packet[6] = calADC12_gain;
    	    */
    				packetLength = packetSize (wMBusPacketC[0]);
    				encodeTXPacket(TXbytes, wMBusPacketC, packetLength); //CRC16 calculation included
    	            for (i = 0; i < packetLength; i++)  {
    	              packet[i] = TXbytes[i];               //wMBusPacketC[i-2];  //original: random data
    	               //MS 2. Oct'15: use LMT70 data
    	            }
    	}
    	else if (mode == WMBUS_TMODE) { //0x04 1st Byte??
    				TXpacket[13] = DIF_FIELD; //0x0B; need overwrite due to TXpacket->TXbytes->TXpacket handling
    				TXpacket[14] = VIF_FIELD; //0x13;
    				encodeTXPacket(TXbytes, TXpacket, TXdataLength);

    				//calculate T-mode TXpacket length
    				packetLength = packetSize (TXbytes[0]); //works for S-mode packet with Manchester in HW
    				encodeTXBytesTmode(TXpacket, TXbytes, packetLength);
    				TmodeByteLength = byteSize(2, 1, packetLength);
    				for (i = 0; i < TmodeByteLength; i++)  {
    	                          packet[i] = TXpacket[i];               //wMBusPacketC[i-2];  //original: random data
    	                          //MS 2. Oct'15: use LMT70 data
    				}
    			//	cc112xSpiWriteTxFifo(TXpacket, TmodeByteLength);
    	}
    	else if (mode == WMBUS_SMODE) {
    			    encodeTXPacket(TXbytes, TXpacket, TXdataLength);
    			    packetLength = packetSize (TXpacket[0]); //works for S-mode packet with Manchester in HW
    				//cc112xSpiWriteTxFifo(TXbytes, packetLength);
    	                    for (i = 0; i < packetLength; i++)  {
    	                          packet[i] = TXbytes[i];               //wMBusPacketC[i-2];  //original: random data
    	                          //MS 2. Oct'15: use LMT70 data
    	                    }
    	}


        /* 2. Set absolute TX time to utilize automatic power management */
        time += PACKET_INTERVAL;
        RF_cmdPropTx.startTime = time;

        /* 3. Send packet */
        RF_EventMask result = RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropTx, RF_PriorityNormal, NULL, 0);
        if (!(result & RF_EventLastCmdDone))
        {
            /* Error */
            while(1);
        }

  //      PIN_setOutputValue(pinHandle, Board_LED1,!PIN_getOutputValue(Board_LED1));
    }
}

void taskFxn(UArg a0, UArg a1) {

    // Initialize the Sensor Controller
    scifOsalInit();
    scifOsalRegisterCtrlReadyCallback(scCtrlReadyCallback);
    scifOsalRegisterTaskAlertCallback(scTaskAlertCallback);
    scifInit(&scifDriverSetup);

    // Set the Sensor Controller task tick interval to 1 second (was 1/16 second)
    // scifStartRtcTicksNow(0x00010000 * 1); // 1.Feb'17: 0.067mW @3.6V @ 1Hz with FET430 EnergyTrace, R18 removed
    //24.2.17: new PCB with 200uH, R18 off: 5,2uA @ 1 Hz and 2 voltage dividers always on for L1 and L2
    //8.3.2017: 2.2 uA without R-dividers R7/8 and R9/10 at 1Hz, .. with RF every 20secs

    scifStartRtcTicksNow(0x00010000 / 16); // 3.26uA are 2 x 1M/1.2M divider at 3.6V VDD
    //24.2.17: 8.2uA but with 2 x voltage dividers of 1.627uA each = 3.26uA extra
    //8.3.2017: 5.3uA without R-dividers R7/8 and R9/10 at 16Hz and 305 uA with RF every 20secs


    //Resistor divider at the LC output: 3.3V thru 1M+1.2M = 1.5uA
    //EnergyTrace on Black MSP430 FET connected to "VDD to EM" pin and "GND" Pin on 6-pin Header
    //scifStartRtcTicksNow(0x00010000 / 64); // 3.26uA are 2 x 1M/1.2M)
    //8.3.2017: 20.2 uA without R-dividers R7/8 and R9/10 at 16Hz and 412 uA with RF every 20secs


    //scifStartRtcTicksNow(0x00010000 / 100); // 3.26uA are 2 x 1M/1.2M
    //24.2.17: 34.8uA but with 2 x voltage dividors of 1.627uA each = 3.26uA extra
    //8.3.2017: 31 uA without R-dividers R7/8 and R9/10 at 16Hz and 337 uA with RF every 20secs


    //scifStartRtcTicksNow(0x00010000 / 128); //
    //8.3.2017: 36.3 uA without R-dividers R7/8 and R9/10 at 16Hz and 214 to 236 uA uA with RF every 20secs

    //scifStartRtcTicksNow(0x00010000 / 160); //
    //8.3.2017: 50uA without R-dividers R7/8 and R9/10 at 16Hz and 227 to 258uA with RF every 20secs

   // scifStartRtcTicksNow(0x00010000 / 200); //
    //8.3.2017: 62.8uA without R-dividers R7/8 and R9/10 at 16Hz and 227 to 258uA with RF every 20secs

    // Start the Sensor Controller tasks (not to be confused with OS tasks) at the same time using | operator;
    scifStartTasksNbl(BV(SCIF_LCSENSE_TASK_ID));
    // no SensorTask started, no RtcTicks: 0.062mW @3.6V (RF periodic TX is working), 17.22uA (TIDA-01228 attached)
    //Read out ADC calibration values for Analog Output from LMT70 to CC1310 ADC
   // calADC12_offset = AUXADCGetAdjustmentOffset(AUXADC_REF_FIXED);
   // calADC12_gain = AUXADCGetAdjustmentGain(AUXADC_REF_FIXED);


    // Main loop
    while (1) {
        // Wait for an ALERT callback
        Semaphore_pend(semHandle, BIOS_WAIT_FOREVER);

        // Clear the ALERT interrupt source
        scifClearAlertIntSource();	//

        // Get the alert events
        uint32_t bvAlertEvents = scifGetAlertEvents();
/*
        if (bvAlertEvents & BV(SCIF_CAP_TOUCH_BUTTON_TASK_ID)) {

             	pcbTouchCounter++;	 //MS: 10.2.16 CapTouch event is detected reliably every 1sec when PCB touched
             	//CapTouch event detected, activate LCD display
             	//pass Semaphore to HeartBeatFxn()
             	Semaphore_post(semLCDHandle);}

        if (bvAlertEvents & BV(SCIF_LMT70_TASK_ID)) {
			#ifdef SCIF_FLOW_METER_DIO_OP_DIFF
			// Print TDC values
	 //       sprintf(pLine + 0,  "%4d ",  scifTaskData.lmt70powerRead.output.pTdcValues[0]);
	 //       sprintf(pLine + 5,  "%4d\n", scifTaskData.lmt70powerRead.output.pTdcValues[1]);
			#else

		   lmt70_HeatSensor = AUXADCAdjustValueForGainAndOffset(scifTaskData.lmt70.output.adcVal1, calADC12_gain, calADC12_offset);
		   lmt70_RoomSensor = AUXADCAdjustValueForGainAndOffset(scifTaskData.lmt70.output.adcVal2, calADC12_gain, calADC12_offset);

			//Linear Interpolation method for LMT70 temp. calculation
			// Thus the final equation used to calculate the measured temperature (TM) in the range between 20C and 30C is: TM = m  VTAO + b
			// TM = 0.193 C/mV  VTAO + 212.009C
			// where VTAO is in mV and TM is in C.
		   temp1_LMT70lin = -0.193 * lmt70_HeatSensor * 1440 / 4095 + 212.009;
		   temp2_LMT70lin = -0.193 * lmt70_RoomSensor * 1440 / 4095 + 212.009;

		   temp1_LMT70dual = a * lmt70_HeatSensor * lmt70_HeatSensor * 1440*1440/(4095*4095) + b * lmt70_HeatSensor * 1440 / 4095 + c;
		   temp2_LMT70dual = a * lmt70_RoomSensor * lmt70_RoomSensor * 1440*1440/(4095*4095)+  b * lmt70_RoomSensor * 1440 / 4095 + c;

		   #endif
			Semaphore_post(semHandle);
        }
        */

        //LC sensor task generated the Alert
        if (bvAlertEvents & BV(SCIF_LCSENSE_TASK_ID)) {
        	Semaphore_post(semLCHandle);

        	discturns = scifTaskData.lcsense.output.effCounter;

           // wMBusPacketC[18] = wMBusPacketC[18] + 1; //(unsigned` char) (ui16wmbusACC_Counter&0x000000FF);

            wMBusPacketC[17] = discturns; //(unsigned char) (ui16wmbusACC_Counter&0x000000FF);
            wMBusPacketC[16] = discturns >> 8;
        }

        // Acknowledge the alert event
        scifAckAlertEvents();
    }

}
/*
 *  ======== main ========
 */
int main(void)
{
    // MS 7.3.16
    Task_Params taskParams;

    //FG 25.10.16
    //One immediate M3 wake up event to allow recharge period algorithm to find an optimal recharge period
    /* Construct BIOS Objects*/
    Clock_Params clkParams;
   Clock_Params_init(&clkParams);
   clkParams.period = 500000/Clock_tickPeriod;
   clkParams.startFlag = TRUE;
    /* Construct a one-shot Clock Instance*/
   Clock_construct(&clkStruct, (Clock_FuncPtr)clkFxn, 1500000/Clock_tickPeriod, &clkParams);

    // Optional: Prevent the system from entering standby to allow debugging
#if DISABLE_LOW_POWER
    Power_setConstraint(Power_SB_DISALLOW);
    Power_setConstraint(Power_IDLE_PD_DISALLOW);
#endif

	/* Call board init functions. */
    Board_initGeneral();

    // Configure the OS task
    Task_Params_init(&taskParams);
    taskParams.stack = myTaskStack;
    taskParams.arg0 = 1000000  / Clock_tickPeriod;  // 1ms
    taskParams.stackSize = sizeof(myTaskStack);
    taskParams.priority = 2;
    Task_construct(&myTask, taskFxn, &taskParams, NULL);

    // Create the semaphore used to wait for Sensor Controller ALERT events
    Semaphore_Params semParams;
    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    Semaphore_construct(&semScTaskAlert, 0, &semParams);
    semHandle = Semaphore_handle(&semScTaskAlert);

    // Create the semaphore used for the LCD
    Semaphore_construct(&semLCDOn, 0, &semParams);
    semLCDHandle = Semaphore_handle(&semLCDOn);

    //create semaphore for LC Task
    Semaphore_construct(&semLC, 0, &semParams);
    semLCHandle = Semaphore_handle(&semLC);

      /* SysMin will only print to the console when you call flush or exit */
      System_flush();

    /* Initialize task */
   TxTask_init(ledPinHandle); //FG: Disable to only measure LCD
    //MS Set LCD to OFF

    //Semaphore_post(semLCDHandle); //FG: Enable when TxTask_init(ledPinHandle) commented out

    /* Start BIOS */
    BIOS_start();

    return (0);
}


//FG 25.10.16
//Immediate M3 wake up event to allow re-charge period algorithm to find an optimal re-charge period

Void clkFxn(UArg arg0)
{
    static int counter = 0;
    if (++counter < 50)
    {
        clkHandle = Clock_handle(&clkStruct);
        Clock_stop(clkHandle);
    }
}  // 21.3.17: This loop does not require much power . The next time the OS wakes up it remembers
// that it does not need a lot of power. Consumption is lowered about ???? 2uA ????

